@@ -8,13 +8,14 @@ from collections import defaultdict
 
 from .packages import nltk
 from .decorators import cached_property
-from .utils import lowerstrip, strip_punc, PUNCTUATION_REGEX
+from .utils import lowerstrip, PUNCTUATION_REGEX
 from .inflect import singularize as _singularize, pluralize as _pluralize
 from .en import sentiment as _sentiment
 from .mixins import ComparableMixin
-from .compat import text_type, string_types, unicode
+from .compat import string_types, unicode
 from .np_extractors import BaseNPExtractor, FastNPExtractor
 from .taggers import BaseTagger, PatternTagger
+from .tokenizers import BaseTokenizer, WordTokenizer, SentenceTokenizer
 from .exceptions import MissingCorpusException
 
 
@@ -34,7 +35,7 @@ class Word(unicode):
         self.pos_tag = pos_tag
 
     def __repr__(self):
-        return "Word('{0}')".format(self.string)
+        return repr(self.string)
 
     def __str__(self):
         return self.string
@@ -117,27 +118,31 @@ class BaseBlob(ComparableMixin):
     '''An abstract base class that all text.blob classes will inherit from.
     Includes words, POS tag, NP, and word count properties. Also includes
     basic dunder and string methods for making objects like Python strings.
+
+    :param text: A string.
+    :param tokenizer: (optional) A tokenizer instance. If ``None``, defaults to :class:`WordTokenizer() <text.tokenizers.WordTokenizer>`.
+    :param np_extractor: (optional) An NPExtractor instance. If ``None``, defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
+    :param pos_tagger: (optional) A Tagger instance. If ``None``, defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
+    :param clean_html: (optional) Remove HTML markup from ``text``.
     '''
 
     np_extractor = FastNPExtractor()
     pos_tagger = PatternTagger()
+    tokenizer = WordTokenizer()
 
-    def __init__(self, text, np_extractor=None, pos_tagger=None):
-        '''Create a blob-like object.
-
-        Arguments:
-        - text: A string, the text.
-        - np_extractor: An instance of an noun phrase extractor class found in
-            text.np_extractor. Defaults to the FastNPExtractor.
-        - pos_tagger: An instance of a POS tagger found in text.taggers.
-            Defaults to the PatternTagger.
-         '''
+    def __init__(self, text, tokenizer=None,
+                pos_tagger=None, np_extractor=None, clean_html=False):
         if type(text) not in string_types:
             raise TypeError('The `text` argument passed to `__init__(text)` '
                             'must be a string, not {0}'.format(type(text)))
-        self.raw = text
-        self.string = text
-        self.stripped = lowerstrip(text)
+        self.raw = self.string = text if not clean_html else nltk.clean_html(text)
+        self.stripped = lowerstrip(self.raw, all=True)
+        # tokenizer may be a textblob or an NLTK tokenizer
+        if (tokenizer is not None and
+                not (isinstance(tokenizer, BaseTokenizer) or
+                    isinstance(tokenizer, nltk.tokenize.api.TokenizerI))):
+            raise ValueError("tokenizer must be an instance of BaseTokenizer")
+        self.tokenizer = tokenizer if tokenizer else BaseBlob.tokenizer
         if (np_extractor is not None and
                 not isinstance(np_extractor, BaseNPExtractor)):
             raise ValueError("np_extractor must be an instance of BaseNPExtractor")
@@ -147,21 +152,28 @@ class BaseBlob(ComparableMixin):
             raise ValueError("pos_tagger must be an instance of BaseTagger")
         self.pos_tagger = pos_tagger if pos_tagger else BaseBlob.pos_tagger
 
-    def _tokenize(self):
-        '''Tokenizes the blob into words.'''
-        return nltk.tokenize.word_tokenize(self.raw)
-
-    # @cached_property
-    # def np_extractor(self):
-    #     from .np_extractor import NPExtractor
-    #     return NPExtractor()
-
     @cached_property
     def words(self):
-        '''Returns list of word tokens.
+        '''Return a list of word tokens. This excludes punctuation characters.
+        If you want to include punctuation characters, access the ``tokens``
+        property.
         '''
-        return WordList([strip_punc(word) for word in self._tokenize()
-                        if strip_punc(word)])  # Excludes punctuation
+        return WordList(WordTokenizer().tokenize(self.raw, include_punc=False))
+
+    @cached_property
+    def tokens(self):
+        '''Return a list of tokens, using this blob's tokenizer object
+        (defaults to :class:`WordTokenizer <text.tokenizers.WordTokenizer>`).
+        '''
+        return WordList(self.tokenizer.tokenize(self.raw))
+
+    def tokenize(self, tokenizer=None):
+        '''Return a list of tokens, using ``tokenizer``.
+
+        :param tokenizer: (optional) A tokenizer object. If None, defaults to this blob's default tokenizer.
+        '''
+        t = tokenizer if tokenizer is not None else self.tokenizer
+        return WordList(t.tokenize(self.raw))
 
     @property
     def sentiment(self):
@@ -209,6 +221,16 @@ class BaseBlob(ComparableMixin):
             counts[phrase] += 1
         return counts
 
+    def ngrams(self, n=3):
+        '''Return a list of n-grams (tuples of n successive words) for this
+        blob.
+        '''
+        if n <= 0:
+            return []
+        grams = [WordList(self.words[i:i+n])
+                            for i in range(len(self.words) - n + 1)]
+        return grams
+
     def __repr__(self):
         '''Returns a string representation for debugging.'''
         class_name = self.__class__.__name__
@@ -273,7 +295,7 @@ class BaseBlob(ComparableMixin):
         Arguments:
         - `other`: a string or a text object
         '''
-        if isinstance(other, text_type):
+        if type(other) in string_types:
             return TextBlob(str(self) + other)
         elif isinstance(other, BaseBlob):
             return TextBlob(str(self) + str(other))
@@ -368,22 +390,33 @@ class BaseBlob(ComparableMixin):
 class TextBlob(BaseBlob):
 
     """A general text block, meant for larger bodies of text (esp. those
-    containing sentences.
+    containing sentences). Inherits from :class:`BaseBlob <BaseBlob>`.
+
+    :param text: A string.
+    :param np_extractor: (optional) An NPExtractor instance. If ``None``, defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
+    :param pos_tagger: (optional) A Tagger instance. If ``None``, defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
+    :param clean_html: (optional) Remove HTML markup from ``text``.
     """
 
-    def __init__(self, text, *args, **kwargs):
-        '''Initialize a textblob.
-
-        Arguments:
-        - `text`: a string
-        '''
-        super(TextBlob, self).__init__(text, *args, **kwargs)
-
     @cached_property
     def sentences(self):
-        '''List of Sentence objects.'''
+        '''Return list of :class:`Sentence <Sentence>` objects.'''
         return TextBlob.create_sentence_objects(self.raw)
 
+    @cached_property
+    def words(self):
+        '''Return a list of word tokens. This excludes punctuation characters.
+        If you want to include punctuation characters, access the ``tokens``
+        property.
+        '''
+        # NLTK's word tokenizer expects sentences as input, so tokenize the
+        # blob into sentences before tokenizing to words
+        words = []
+        for sent in self.sentences:
+            words.extend(WordTokenizer().tokenize(str(sent), include_punc=False))
+        return WordList(words)
+
+
     @property
     def raw_sentences(self):
         '''List of strings, the raw sentences in the blob.'''
@@ -394,21 +427,21 @@ class TextBlob(BaseBlob):
         '''Returns a list of each sentences dict representation.'''
         return [sentence.dict for sentence in self.sentences]
 
-    @property
-    def json(self):
+    def json(self, *args, **kwargs):
         '''Returns a json representation of this blob.'''
-        return json.dumps(self.serialized)
+        return json.dumps(self.serialized, *args, **kwargs)
 
     @staticmethod
     def create_sentence_objects(blob):
         '''Returns a list of Sentence objects given
         a list of sentence strings. Attempts to handle sentences that
-        have more than one puntuation mark at the end of the sentence.
+        have more than one punctuation mark at the end of the sentence.
         Examples: "An ellipses is no problem..." or "This is awesome!!!"
         '''
+        sent_tokenizer = SentenceTokenizer()
         sentence_objects = []
         try:
-            sentences = nltk.tokenize.sent_tokenize(blob)  # List of raw sentences
+            sentences = sent_tokenizer.tokenize(blob)  # List of raw sentences
         except LookupError:
             raise MissingCorpusException()
         # if there is only one sentence or string of text
@@ -445,23 +478,20 @@ class TextBlob(BaseBlob):
 
 class Sentence(BaseBlob):
 
-    '''A sentence within a TextBlob.'''
+    '''A sentence within a TextBlob. Inherits from :class:`BaseBlob <BaseBlob>`.
 
-    def __init__(self, sentence, start_index=0, end_index=None):
-        '''Initialize a Sentence.
+    :param sentence: A string, the raw sentence.
+    :param start_index: An int, the index where this sentence begins
+                        in a TextBlob. If not given, defaults to 0.
+    :param end_index: An int, the index where this sentence ends in
+                        a TextBlob. If not given, defaults to the
+                        length of the sentence - 1.
+    '''
 
-        Arguments:
-        - sentence: A string, the raw sentence.
-        - start_index: An int, the index where this sentence begins
-                            in a TextBlob. If not given, defaults to 0.
-        - end_index: An int, the index where this sentence ends in
-                            a TextBlob. If not given, defaults to the
-                            length of the sentence - 1.
-        '''
-        super(Sentence, self).__init__(sentence)
+    def __init__(self, sentence, start_index=0, end_index=None, *args, **kwargs):
+        super(Sentence, self).__init__(sentence, *args, **kwargs)
         self.start = self.start_index = start_index
-        self.end = self.end_index = (end_index if end_index else len(sentence)
-                                     - 1)
+        self.end = self.end_index = end_index if end_index else len(sentence) - 1
 
     @property
     def dict(self):
@@ -474,3 +504,54 @@ class Sentence(BaseBlob):
             'noun_phrases': self.noun_phrases,
             'sentiment': self.sentiment,
         }
+
+class Blobber(object):
+
+    '''A factory for TextBlobs that all share the same tagger,
+    tokenizer, and np_extractor.
+
+    Usage:
+
+        >>> from text.blob import Blobber
+        >>> from text.taggers import NLTKTagger
+        >>> from text.tokenizers import SentenceTokenizer
+        >>> tb = Blobber(pos_tagger=NLTKTagger(), tokenizer=SentenceTokenizer())
+        >>> blob1 = tb("This is one blob.")
+        >>> blob2 = tb("This blob has the same tagger and tokenizer.")
+        >>> blob1.pos_tagger == blob2.pos_tagger
+        True
+
+    :param tokenizer: A tokenizer. Default: :class:`WordTokenizer <text.tokenizers.WordTokenizer>`.
+    :param pos_tagger: A POS tagger. Default: :class:`PatternTagger <text.taggers.PatternTagger>`.
+    :param np_extractor: A NP extractor. Default: :class:`FastNPExtractor <text.np_extractors.FastNPExtractor>`.
+    '''
+
+    np_extractor = FastNPExtractor()
+    pos_tagger = PatternTagger()
+    tokenizer = WordTokenizer()
+
+    def __init__(self, tokenizer=None, pos_tagger=None, np_extractor=None):
+        # tokenizer may be a textblob or an NLTK tokenizer
+        if (tokenizer is not None and
+                not (isinstance(tokenizer, BaseTokenizer) or
+                    isinstance(tokenizer, nltk.tokenize.api.TokenizerI))):
+            raise ValueError("tokenizer must be an instance of BaseTokenizer")
+        self.tokenizer = tokenizer if tokenizer else Blobber.tokenizer
+        if (np_extractor is not None and
+                not isinstance(np_extractor, BaseNPExtractor)):
+            raise ValueError("np_extractor must be an instance of BaseNPExtractor")
+        self.np_extractor = np_extractor if np_extractor else Blobber.np_extractor
+        if (pos_tagger is not None and
+                not isinstance(pos_tagger, BaseTagger)):
+            raise ValueError("pos_tagger must be an instance of BaseTagger")
+        self.pos_tagger = pos_tagger if pos_tagger else Blobber.pos_tagger
+
+    def __call__(self, text):
+        '''Return a new TextBlob object with this Blobber's ``np_extractor``,
+        ``pos_tagger``, and ``tokenizer``.
+        '''
+        return TextBlob(text, tokenizer=self.tokenizer, pos_tagger=self.pos_tagger,
+            np_extractor=self.np_extractor)
+
+
+
